// Importing specific functions and objects from the 'd3' library.
import {
  select,
  csv,
  scaleLinear,
  max,
  scaleBand,
  axisLeft,
  axisBottom,
  format,
  tip,
  scaleOrdinal,
  mouse,
} from 'd3';
// Setting Title and label for the chart.
const titleText =
  'Top food Stores in Dallas in Each Places';
const xAxisLabelText =
  'Number of 4-starts-or-above Stores in Dallas';
// Selecting the SVG element defined in HTML document and storing to set other parameters.
const svg = select('svg');
// Converting the width and height attributes of the SVG element to numerical.
const width = +svg.attr('width');
const height = +svg.attr('height');
// Defining the 'render' function, which takes 'data' as input and is responsible for creating the stacked bar chart.
const render = (data) => {
  //const xValue = d => +d['count'];
  const yValue = (d) => d.city;
  //const cValue = d => d.food;
  const margin = {
    top: 50,
    right: 40,
    bottom: 77,
    left: 120,
  };
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
  // Extracting the list of food stores from the data columns.
  const food = data.columns.slice(1);
  // Creating a map of unique cities from the 'city' column in the data
  const city = d3.map(data, (d) => d.city);
  // Stacking the data of 'food' stores to create stacked bar chart.
  const stackedData = d3.stack().keys(food)(data);
  //setting the colorscale for the bar chart.
  const colorScale = [
    '#e6e600',
    '#ffa31a',
    '#cc5200',
    '#33bbff',
    '#8cff1a',
    '#bf6b7c',
    '#29a3a3',
    '#ff0066',
    '#00ffff',
    '#fc8383',
  ];
  const color = scaleOrdinal()
    .domain(food)
    .range(colorScale);

//Setting parameters for x-axis labels
  const xScale = scaleLinear()
    .domain([0, 300])
    .range([0, innerWidth]);
//Setting parameters for y-axis labels
  const yScale = scaleBand()
    .domain(city)
    .range([0, innerHeight])
    .padding(0.1);
// Appending a 'g' element to the SVG to create a container for the chart and applying margins to it.
  const g = svg
    .append('g')
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`
    );

  const xAxis = axisBottom(xScale).tickSize(-innerHeight);
  // Creating and appending the y-axis to the 'g' element.
  g.append('g')
    .call(axisLeft(yScale))
    .selectAll('.domain, .tick line')
    .style("font-size", "10px")
    .remove();
  // Creating and appending the x-axis to the 'g' element.
  const xAxisG = g
    .append('g')
    .call(xAxis)
    .style("font-size", "12px")
    .attr('transform', `translate(0,${innerHeight})`);

  xAxisG.select('.domain').remove();
  // Adding a label to the x-axis.
  xAxisG
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 65)
    .attr('x', innerWidth / 2)
    .attr('fill', 'black')
    .text(xAxisLabelText);

  // Adding the tool tip to the body
  const tooltip = select('body').append('div').attr('class', 'toolTip');
  
  tooltip
    .append('rect')
    .attr('width', 60)
    .attr('height', 20)
    .attr('fill', 'none')
    .style('opacity', 0.5);

  tooltip
    .append('text')
    .attr('x', 30)
    .attr('dy', '1.2em')
    .style('text-anchor', 'middle')
    .attr('font-size', '12px')
    .attr('font-weight', 'bold');
  
  // Creating and appending the stacked bars to the chart using the data and scales.
  g.append('g')
    .selectAll('g')
    .data(stackedData)
    .enter()
    .append('g')
    .attr('fill', (d) => color(d.key))
    .selectAll('rect')
    .data((d) => d)
    .enter()
    .append('rect')
    .attr('y', (d) => yScale(d.data.city))
    .attr('x', (d) => xScale(d[0]))
    .attr('width', (d) => xScale(d[1]) - xScale(d[0]))
    .attr('height', yScale.bandwidth())
    .on('mouseover', function (event, d) {
      d3.select(this).attr('opacity', 0.5);
    })
    .on('mousemove', function (event, d) {
      const foodtype = d3.select(this.parentNode).datum()
        .key;
      const foodcount = d.data[foodtype];
      tooltip
        .style('left', event.pageX - 50 + 'px')
        .style('top', event.pageY - 70 + 'px')
        .style('display', 'inline-block')
        .html(foodtype + ' Stores_Count: ' + foodcount);
    })
    .on('mouseout', function (d) {
      d3.select(this).attr('opacity', 1);
      tooltip.style('display', 'none');
    });
  // Adding the chart title.
  g.append('text')
    .attr('class', 'title')
    .attr('y', -10)
    .text(titleText);
  // Adding a subheading for the food stores.
  g.append('text')
    .attr('text-anchor', 'right')
    .attr('x', 670)
    .attr('y', 100)
    .style('font-size', '20px')
    .style('font-weight', 'bold')
    .text('Food Stores');
// Adding legends for each 'food' type.
  g.selectAll('legendsqr')
    .data(food)
    .enter()
    .append('rect')
    .attr('x', 660)
    .attr('y', function (d, i) {
      return 110 + i * 26;
    })
    .attr('width', 15)
    .attr('height', 15)
    .style('fill', function (d) {
      return color(d);
    });

  g.selectAll('legendLabels')
    .data(food)
    .enter()
    .append('text')
    .attr('x', 700)
    .attr('y', function (d, i) {
      return 118 + i * 26;
    })
    .style('fill', function (d) {
      return color(d);
    })
    .text(function (d) {
      return d;
    })
    .attr('text-anchor', 'left')
    .style('alignment-baseline', 'middle')
    .style('font-size', '18px')
    .style('font-weight', 'bold');
};
// Loading the data from the 'data.csv' file and calling the 'render' function to create the chart.
csv('data.csv').then((data) => {
  render(data);
});